import * as extractors from "../featureExtractors";
import { MeydaSignal, MeydaWindowingFunction } from "../main";
import { A } from "ts-toolbelt";
import { MatchTupleLength } from "./match-tuple-length";
export declare type MeydaAudioFeature = keyof typeof extractors;
declare type ExtractorMap = typeof extractors;
declare type ReturnTypesOf<T extends Record<string, (...args: any) => any>> = {
    [K in keyof T]: ReturnType<T[K]>;
};
interface MeydaConfigurationOptions {
    sampleRate: number;
    bufferSize: number;
    windowingFunction: MeydaWindowingFunction | "rect";
    mfccCoefficients: number;
    chromaBands: number;
    melBands: number;
    barkBands: number;
}
declare type ConcreteResultType<T extends MeydaAudioFeature> = {
    [ExtractorName in T]: ReturnTypesOf<ExtractorMap>[ExtractorName];
};
declare type MaybePartialResult<T extends V extends readonly (infer K)[] ? K : V extends (infer K)[] ? K : V, U extends readonly MeydaSignal[] | MeydaSignal, V extends readonly MeydaAudioFeature[] | MeydaAudioFeature> = MatchTupleLength<U, V extends T[] ? A.Compute<Partial<ConcreteResultType<MeydaAudioFeature>>, "flat"> : A.Compute<ConcreteResultType<T>, "flat">>;
/**
 * # Configure an extractor
 *
 * Configure an extractor function with options that are shared throughout the
 * extractor function's lifetime. The options pertain to the type of audio that
 * the extractor expects - e.g. the sample rate, the consistent windowing
 * function to apply, etc.
 *
 * @param options Configuration used across all feature extractions made with
 * resulting interface
 
 * @returns A function that takes a set of requested audio features and one
 * buffer of audio, and returns the requested audio features extracted from that
 * audio.
 */
export declare function configureMeyda(options?: Partial<MeydaConfigurationOptions>): <T extends V extends readonly (infer K)[] ? K : V extends (infer K_1)[] ? K_1 : V, U extends MeydaSignal | readonly MeydaSignal[], V extends "amplitudeSpectrum" | "chroma" | "complexSpectrum" | "energy" | "loudness" | "mfcc" | "perceptualSharpness" | "perceptualSpread" | "powerSpectrum" | "rms" | "spectralCentroid" | "spectralFlatness" | "spectralFlux" | "spectralKurtosis" | "spectralRolloff" | "spectralSkewness" | "spectralSlope" | "spectralSpread" | "zcr" | "buffer" | readonly ("amplitudeSpectrum" | "chroma" | "complexSpectrum" | "energy" | "loudness" | "mfcc" | "perceptualSharpness" | "perceptualSpread" | "powerSpectrum" | "rms" | "spectralCentroid" | "spectralFlatness" | "spectralFlux" | "spectralKurtosis" | "spectralRolloff" | "spectralSkewness" | "spectralSlope" | "spectralSpread" | "zcr" | "buffer")[] | ("amplitudeSpectrum" | "chroma" | "complexSpectrum" | "energy" | "loudness" | "mfcc" | "perceptualSharpness" | "perceptualSpread" | "powerSpectrum" | "rms" | "spectralCentroid" | "spectralFlatness" | "spectralFlux" | "spectralKurtosis" | "spectralRolloff" | "spectralSkewness" | "spectralSlope" | "spectralSpread" | "zcr" | "buffer")[]>(feature: V, signal: U) => MatchTupleLength<U, V extends T[] ? {
    amplitudeSpectrum?: Float32Array | undefined;
    chroma?: number[] | undefined;
    complexSpectrum?: {
        real: number[];
        imag: number[];
    } | undefined;
    energy?: number | undefined;
    loudness?: {
        specific: Float32Array;
        total: number;
    } | undefined;
    mfcc?: number[] | undefined;
    perceptualSharpness?: number | undefined;
    perceptualSpread?: number | undefined;
    powerSpectrum?: Float32Array | undefined;
    rms?: number | undefined;
    spectralCentroid?: number | undefined;
    spectralFlatness?: number | undefined;
    spectralFlux?: number | undefined;
    spectralKurtosis?: number | undefined;
    spectralRolloff?: number | undefined;
    spectralSkewness?: number | undefined;
    spectralSlope?: number | undefined;
    spectralSpread?: number | undefined;
    zcr?: number | undefined;
    buffer?: MeydaSignal | undefined;
} : ConcreteResultType<T> extends import("ts-toolbelt/out/Misc/BuiltIn").BuiltIn ? import("ts-toolbelt/out/Misc/BuiltIn").BuiltIn & ConcreteResultType<T> : ConcreteResultType<T> extends any[] ? any[] & ConcreteResultType<T> extends Record<A.Key, any>[] ? { [K_2 in keyof (Record<A.Key, any>[] & any[] & ConcreteResultType<T>)[number]]: (Record<A.Key, any>[] & any[] & ConcreteResultType<T>)[number][K_2]; }[] : any[] & ConcreteResultType<T> : ConcreteResultType<T> extends readonly any[] ? readonly any[] & ConcreteResultType<T> extends readonly Record<A.Key, any>[] ? readonly { [K_3 in keyof (readonly Record<A.Key, any>[] & readonly any[] & ConcreteResultType<T>)[number]]: (readonly Record<A.Key, any>[] & readonly any[] & ConcreteResultType<T>)[number][K_3]; }[] : readonly any[] & ConcreteResultType<T> : { [K_4 in keyof ConcreteResultType<T>]: ConcreteResultType<T>[K_4]; }>;
/**
 * Returns a Meyda extractor function with configured audio features.
 *
 * @param features
 * @param extractor
 * @returns
 */
export declare function curryMeyda<T extends V extends readonly (infer K)[] ? K : V extends (infer K)[] ? K : V, U extends readonly MeydaSignal[] | MeydaSignal, V extends MeydaAudioFeature[] | readonly MeydaAudioFeature[] | MeydaAudioFeature>(features: V, extractor: (feature: V, signal: U) => MaybePartialResult<T, U, V>): (signal: U) => MaybePartialResult<T, U, V>;
/**
 * ### Configure an extractor with options and audio features
 *
 * Configure a Meyda extractor function with options and audio features at once.
 *
 * @param features
 * @param options
 * @returns
 */
export declare function configureMeydaWithExtractors<T extends V extends readonly (infer K)[] ? K : V extends (infer K)[] ? K : V, U extends readonly MeydaSignal[] | MeydaSignal, V extends MeydaAudioFeature[] | readonly MeydaAudioFeature[] | MeydaAudioFeature>(features: V, options?: Partial<MeydaConfigurationOptions>): (signal: U) => MaybePartialResult<T, U, V>;
export {};
